/******************************************************************************

*****************************************************************************/


#include "predef.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <netinterface.h>

#define DHCPTEST_DEBUG  1


extern "C" {
void UserMain(void * pd);
}

// Reference pointer to name for DHCP and DNS
extern const char *pDHCPOfferName;

// Application name that will appear in IPSetup
const char * AppName="DhcpManual";

// Since we want to be able to start and stop DHCP, we need to create out
// own DHCP Object instead of using the GetDHCPAdderss() function which
// handles this automatically.
DhcpObject *pDhcpObj;

// Variable to indicate if appliction tried to obtain runtime IP settings
// from a DHCP server.
bool AssignedDHCP = FALSE;


/*-------------------------------------------------------------------
 MyStartDHCP
 Most applications can just use the GetDHCPAddress() function at boot
 time to enable the DHCP Client. If you require more control over the
 DHCP service, such as starting and stopping, then a few lines of
 code are required to create a DhcpObject and check a semaphore to
 determine if the DHCP request was successful. This is essentially
 what the GetDHCPAddress() function does.
 -------------------------------------------------------------------*/
void MyStartDHCP( int Interface )
{
   InterfaceBlock *ib = GetInterFaceBlock( Interface );  // Get interface data

   // The following lines of code are essentially what the GetDHCPAddress() function
   // does to make the DHCP process easier.
   pDhcpObj = new  DhcpObject( Interface );
   pDhcpObj->StartDHCP();    // Start DHCP

   // Pend on semaphore to verify an address was obtained
   if ( OSSemPend( &( pDhcpObj->NotifySem ), 10 * TICKS_PER_SECOND ) == OS_TIMEOUT ) //Wait 10 seconds
   {
      iprintf("\r\n\r\n*** WARNING ***\r\n");
      iprintf("IP Address was set to 0.0.0.0, and a DHCP server could not be found.\r\n");
      iprintf("Device does not have a valid IP address.\r\n\r\n");
   }
   else
   {
      iprintf("DHCP assigned the following values:\r\n");
      iprintf("IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
      iprintf("Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
      iprintf("Gate: "); ShowIP(ib->netIpGate); iprintf("\r\n");
      iprintf("DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
      iprintf("Interface Name: %s\r\n", ib->InterfaceName);
      iprintf("\r\n");
      AssignedDHCP = TRUE;
  }
}


/*-------------------------------------------------------------------
 Display runtime IP values
 This function demonstrates two methods to read the runtime IP
 values:
 1. Using the read-only Interface function calls
 2. Using the GetInterfaceBlock() function to get a pointer to an
    Interface Record.
 -------------------------------------------------------------------*/
void DisplayRuntimeIPSettings(int InterfaceNumber)
{
   iprintf("\r\n\r\n--- RUNTIME IP SETTINGS ---\r\n");
   if ( AssignedDHCP )
      iprintf("Values assigned by DHCP Server\r\n");

   // Display current runtime values using Interface read-only functions
   iprintf("IP runtime settings using Interface Functions for interface %d:\r\n",
            InterfaceNumber);
   iprintf("IP:   "); ShowIP(InterfaceIP(InterfaceNumber)); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(InterfaceMASK(InterfaceNumber)); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(InterfaceGate(InterfaceNumber)); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(InterfaceDNS(InterfaceNumber)); iprintf("\r\n");

   // Display current runtime values by getting a pointer to the Interface Block
   // and accessing it's variables.
   iprintf("\r\nIP runtime settings using GetInterfaceBlock() for interface %d:\r\n",
           InterfaceNumber);
   InterfaceBlock *ib = GetInterFaceBlock(InterfaceNumber);
   iprintf("IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(ib->netIpGate); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
   iprintf("Interface Name: %s\r\n", ib->InterfaceName);
   iprintf("\r\n");
}

extern void PrintDhcpStatus( DhcpObject &dob );

DWORD   ProcessDhcpCommandTaskStack[USER_TASK_STK_SIZE];

/*-------------------------------------------------------------------
 Interactive menu of DHCP commands
 -------------------------------------------------------------------*/
void ProcessDhcpCommandTask( void *pdata )
{
    while (1)
    {
        char c = getchar();
        switch (c)
        {
            case 'a': pDhcpObj->StopDHCP();
                      iprintf("Called StopDHCP()\r\n");
                      break;

            case 'b': pDhcpObj->StartDHCP();
                      iprintf("Called StartDHCP()\r\n");
                      break;

            case 'i': {iprintf("Enter value to add: ");
                        char buf[20];
                        gets(buf);
                        int adder = atoi(buf);
                        iprintf("Adding %d to Secs..\r\n", adder);
                        Secs += (DWORD)adder;
                        }
                      break;

            case 'j': Secs = (DWORD)(pDhcpObj->GetDhcpRenewTime() - 5);
                      iprintf("Secs set to: %ld\r\n", Secs);
                      break;

            case 'k': Secs = (DWORD)(pDhcpObj->GetDhcpRebindTime() - 5);
                      iprintf("Secs set to: %ld\r\n", Secs);
                      break;

            case 'l': Secs = (DWORD)(pDhcpObj->GetDhcpExpirationTime() - 5);
                      iprintf("Secs set to: %ld\r\n", Secs);
                      break;

            case 'r': pDhcpObj->RenewDHCP();
                      break;

            #ifdef DHCPTEST_DEBUG
            case 's': PrintDhcpStatus( (DhcpObject &) *pDhcpObj );
                      break;
            #endif

            case 't': iprintf("\r\nRemaining lease time: %ld\r\n", pDhcpObj->GetRemainingDhcpLeaseTime());
                        iprintf("System Timer: %ld\r\n", Secs);
                      break;

            case 'v': if (pDhcpObj->ValidDhcpLease())
                        iprintf("Lease is valid\r\n");
                      else
                        iprintf("Lease is INVALID\r\n");
                      break;

            default:
            iprintf("\r\nMain Menu\r\n");
            iprintf("---------\r\n");
            iprintf("a = Stop DHCP service manually\r\n");
            iprintf("b = Start DHCP service manually\r\n");
            iprintf("i = Increment Secs system timer\r\n");
            iprintf("j = Set Secs counter to 5 seconds before renew time\r\n");
            iprintf("k = Set Secs counter to 5 seconds before rebind time\r\n");
            iprintf("l = Set Secs counter to 5 seconds before lease exp. time\r\n");
            iprintf("t = Get remaining lease time\r\n");
            #ifdef DHCPTEST_DEBUG
            iprintf("s = Display DHCP Status\r\n");
            #endif
            iprintf("r = DHCP Renew\r\n");
            iprintf("v = Check for valid DHCP lease\r\n");
        }
    }
}


/*-------------------------------------------------------------------
 UserMain task
 -------------------------------------------------------------------*/
void UserMain(void * pd)
{
    InitializeStack();

    pDHCPOfferName = "NetBurner";   // Host name for DNS

    int EthernetInterface = GetFirstInterface();  // Get first interface identifier
    InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data

    if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
    {
       iprintf("\r\nNo static IP address set, attempting DHCP\r\n");
       MyStartDHCP( EthernetInterface );
    }
    else
    {
       iprintf("Static IP Address: ");ShowIP(ib->netIP);iprintf("/r/n");
    }

    OSChangePrio(MAIN_PRIO);
    EnableAutoUpdate();

    iprintf("Application started");

    // Start task to process user keyboard input commands
    if ( OSTaskCreate( ProcessDhcpCommandTask,
                       NULL,
                       ( void * ) &ProcessDhcpCommandTaskStack[USER_TASK_STK_SIZE],
                       ( void * ) ProcessDhcpCommandTaskStack,
                       MAIN_PRIO + 1 ) != OS_NO_ERR )
    {
       iprintf( "*** Error creating task\r\n" );
    }

    while (1)
    {
       PrintDhcpStatus( (DhcpObject &) *pDhcpObj );

       if ( pDhcpObj->ValidDhcpLease() )
          iprintf("ValidDhcpLease() = TRUE\r\n");
       else
          iprintf("ValidDhcpLease() = FALSE\r\n");

       iprintf( "GetRemainingDhcpLeaseTime() = %ld\r\n", pDhcpObj->GetRemainingDhcpLeaseTime());

       OSTimeDly( TICKS_PER_SECOND * 5 );
    }
}



